import idautils, idc

def get_decrypt_func(name="decryptString"):
	for ea, n in idautils.Names():
		if n == name:
			return ea 
	return None 
	
	
def get_call_sites(ea):
	return [x.frm for x in idautils.XrefsTo(ea)]
	

def get_push_value(inst):
	mnem = idc.GetMnem(inst)
	op_type = idc.GetOpType(inst, 0)
	op_val = idc.GetOperandValue(inst, 0)
	
	#push imm8/32
	if  mnem == "push" and op_type == 5:
		return op_val
			
	#push reg
	if  mnem == "push" and op_type == 1:
		#print("push reg")
		min_ea = idaapi.get_func(inst).start_ea
		while inst >= min_ea:
			inst = idc.PrevHead(inst)
			
			mnem = idc.GetMnem(inst)
			op_type1 = idc.GetOpType(inst, 0)
			op_type2 = idc.GetOpType(inst, 1)
			op_val1 = idc.GetOperandValue(inst, 0)
			op_val2 = idc.GetOperandValue(inst, 1)
			
			#mov r32, imm32
			if mnem == "mov" and op_type1 == 1 and op_type2 == 5 and op_val1 == op_val:
				return op_val2
				
			#push imm
			#pop reg32    
			if mnem == "pop" and op_type1 == 1 and op_val1 == op_val:
				prev_inst = idc.PrevHead(inst)
				mnem = idc.GetMnem(prev_inst)
				op_type1 = idc.GetOpType(prev_inst, 0)
				op_val1 = idc.GetOperandValue(prev_inst, 0)
				if mnem == "push" and op_type1 == 5:
					return op_val1
		
	return None

def get_first_push_backward(ea):
    min_ea = idaapi.get_func(ea).start_ea
    
    while ea >= min_ea:
        ea = idc.PrevHead(ea)
        
        if idc.GetMnem(ea) == "push":
            return ea
            
    return None

def decrypt_string(params):
	ptr = params["buffer"] + params["offset"]
	key = idaapi.get_bytes(ptr, params["keylen"])
	data = idaapi.get_bytes(ptr + params["keylen"], params["datalen"])

	return rc4(data, key)
	
def rc4(data, key):
	S = range(256)
	j = 0
	out = ""

	#KSA Phase
	for i in range(256):
		j = (j + S[i] + ord( key[i % len(key)] )) % 256
		S[i] , S[j] = S[j] , S[i]

	#PRGA Phase
	i = j = 0
	for char in data:
		i = ( i + 1 ) % 256
		j = ( j + S[i] ) % 256
		S[i] , S[j] = S[j] , S[i]
		out += chr(ord(char) ^ S[(S[i] + S[j]) % 256])

	return out.replace("\x00", "") 
	
def get_params(sites):

	params = []
	names = ["buffer", "offset", "keylen", "datalen"]
	
	for ea in sites:
		instance = {}
		valid = True
		for i in range(4):
			X = ea
			ea = get_first_push_backward(ea) #, off_push, keylen_push, data_push  = [ea = get_first_push_backward(ea) for _ in range(4)]
			val = get_push_value(ea)

			if val is None:
				print(hex(ea) +  " cannot find " + names[i] + " " + hex(X))
				valid = False
				break
			else:
				instance[names[i]] = val	
				
		if valid and instance["datalen"] <= 400: 		  		
		  params.append(instance)
		elif valid and instance["datalen"] > 400:
		  print("too long: " + hex(ea) + ": " + str(instance))
		  
	
	return params
		
#idc.set_cmt(ea, comment, 1)
sites = get_call_sites(get_decrypt_func())
params = get_params(sites)
strings = [decrypt_string(p) for p in params]

print(strings)